
package com.lordjoe.lib.xml;

import com.lordjoe.utilities.Assertion;
import java.util.*;
import java.lang.reflect.*;
import com.lordjoe.utilities.*;
/**
 * Help class with static methods to set and get properties by name and 
 * access any exposed collections in an object
 * @author Steve Lewis
 * @since july 99
 */

public abstract class ClassAnalyzer
{
    protected static IObjectConverter gConverter;
    
    public static void setObjectConverter(IObjectConverter aConverter)
    {
        gConverter = aConverter;
    }
    
    
    /**
        Table mapping Class to a Hashtable holding
        a map from method name to method
        NOTE - This assumes that there is one and 
        only one public method of a given name 
     */
    protected static Hashtable gClassToAnalysisTable;
    
    /**
        used to invoke getters which have no arguments
    */
    public static final Object[] EMPTY_CALL = new Object[0];
    
    
    /**
        all properties supported by the class
        @param TheClass non-null target class
        @return - non-null properties collection
    */
    public static ClassProperty[] getProperties(Class TheClass)
    {
        ClassAnalysis anal = getAnalysis(TheClass);
        if(anal.m_PropertiesList == null) 
            anal.buildPropertiesList();
        return(anal.m_PropertiesList);
    }
    
    /**
        look uo one property by name
        @param TheClass non-null target class
        @param PropName non-null property name
        @return - non-null properties collection
    */
    public static ClassProperty getClassProperty(Class TheClass,String PropName)
    {
        ClassProperty[] TheProperties = getProperties(TheClass);
        for(int i = 0; i < TheProperties.length; i++) {
            if(TheProperties[i].m_Name.equals(PropName)) {
                return(TheProperties[i]);
            }
        }
        return(null);
    }
    
     /**
        all properties supported by the class
        @param TheClass non-null target class
        @return - non-null properties collection
    */
    public static String[] getPropertyNames(Class TheClass)
    {
       ClassProperty[] props = getProperties(TheClass);
       String[] ret = new String[props.length];
       for(int i = 0; i < ret.length; i++) 
            ret[i] = props[i].m_Name;
       return(ret);
    }
   
    /**
        return all collections supported by the class
        @param TheClass non-null target class
        @return - non-null properties collection
    */
    public static ClassCollection[] getCollections(Class TheClass)
    {
        ClassAnalysis anal = getAnalysis(TheClass);
        if(anal.m_CollectionsList == null) 
            anal.buildCollectionsList();
        return(anal.m_CollectionsList);
    }
    
    
    /**
        return a collection descriptor
        @param TheClass non-null target class
        @param name non-null propertyName - must hand methods get<name>
        @return - possibly null ClassCollection
    */
    public static ClassCollection getCollection(Class TheClass,String name)
    {
        ClassAnalysis anal = getAnalysis(TheClass);
        Hashtable tb = anal.m_Collections;
        ClassCollection ret = (ClassCollection)tb.get(name);
        if(ret != null)
            return(ret);
         // try caseless get
         ret = (ClassCollection)tb.get(name.toUpperCase());
         return(ret);
    }
    /**
        return a collection descriptor
        @param TheObject non-null target object
        @param name non-null propertyName - must hand methods get<name>
        @return - possibly null ClassCollection
    */
    public static ClassCollection getCollection(Object TheObject,String name)
    {
         return(getCollection(TheObject.getClass(),name));
    }
    /**
        add an item to the collection given by name
        @param Target non-null target object
        @param name non-null collection name
        @param added non-null item to add - must be acceptable type
    */
    public static void addCollectionItem(Object TheTarget,String name,Object added)
    {
        ClassCollection col = getCollection(TheTarget,name);
        if(col == null)
            throw new IllegalArgumentException("Collection for " + name + " not found");
        Method TheMethod = col.m_AddMethod;
        try {
            Object[] call = { added };
            TheMethod.invoke(TheTarget,call);         
        }
        catch(IllegalAccessException ex) {
            throw new IllegalArgumentException("IllegalAccessException getter for " + name + " not found");
        }
        catch(InvocationTargetException ex) {
            throw new IllegalArgumentException("InvokationTargetException getter for " + name + " not found");
        }
            
    }
     /**
        return a value for the property of a given property name
        @param Target non-null target object
        @param name non-null collection name
        @param added non-null item to add - must be acceptable type
    */
    public static Enumeration getCollectionItems(Object TheTarget,String name)
    {
        ClassCollection col = getCollection(TheTarget,name);
        if(col == null)
            throw new IllegalArgumentException("Collection for " + name + " not found");
        Method TheMethod = col.m_EnumerateMethod;
        try {
            Object ret = TheMethod.invoke(TheTarget,EMPTY_CALL);
            if(ret instanceof Iterator) {
                return(new IteratorToEnumeration((Iterator)ret));
            }
            return((Enumeration)ret);
        }
        catch(IllegalAccessException ex) {
            throw new IllegalArgumentException("IllegalAccessException getter for " + name + " not found");
        }
        catch(InvocationTargetException ex) {
            throw new IllegalArgumentException("InvokationTargetException getter for " + name + " not found");
        }
            
    }
    /**
    *    return a value for the property of a given property name this
    * call can accept properties of the form customer.address.line1 which
    * will find a property called customer - if non-null will find a property in
    * customer called field which will find a property called line1. If any item 
    * in the chain is null then null is returned
    *    @param Target non-null target object
    *    @param name non-null propertyName - must hand methods set<name>
    *    @return - possibly null propetyValue
    */
    public static Object getProperty(Object TheTarget,String name)
    {
        if(name.indexOf('.') == -1)
            return(getSingleProperty(TheTarget,name));
        String[] items = Util.parseString(name,".");
        Object current = TheTarget;
        for(int i = 0; i < items.length; i++) {
            current = getSingleProperty(current,items[i]);
            if(current == null) 
                return(null);
        }
        return(current);
    }       
   
    /**
    *    set a value for the property of a given property name this
    * call can accept properties of the form customer.address.line1 which
    * will find a property called customer - if non-null will find a property in
    * customer called field which will find a property called line1. If any item 
    * in the chain is null IllegalArgumentException is thrown
    *    @param Target non-null target object
    *    @param name non-null propertyName - must hand methods set<name>
    *    @return - possibly null propetyValue
    */
    protected static Object getSingleProperty(Object TheTarget,String name)
    {
        Class TheClass = TheTarget.getClass();
        Method TheMethod = findGetMethod(TheClass,name);
        if(TheMethod == null) {
            String[] AllProps = getAllProperties(TheClass);
            String Message = "Property getter for " + name + " not found\nVaslid Properties are:\n" +
                Util.buildListString(AllProps, 8);
            System.out.println(Message);
            throw new IllegalArgumentException(Message);
        }
        Object ret = null;
        try {
            ret = TheMethod.invoke(TheTarget,EMPTY_CALL);         
        }
        catch(IllegalAccessException ex) {
            throw new IllegalArgumentException("IllegalAccessException getter for " + name + " not found");
        }
        catch(InvocationTargetException ex1) {
            Throwable RealException = ex1.getTargetException();
            throw new WrapperException("Error getting property " + name,RealException);
        }
        return(ret);
    }
    
    /**
        set the value for the property of a given property name
        @param Target non-null target object
        @param name non-null propertyName - must hand methods set<name>
        @param value possibly null new propetyValue - note if null and there are several set methods 
          any one might be called
        @throws IllegalArgumentException if elements of the chain are null
    */
    public static void setProperty(Object TheTarget,String name,Object value)
    {
        if(name.indexOf('.') == -1) {
            setSingleProperty(TheTarget,name,value);
            return;
        }
            
        String[] items = Util.parseString(name,".");
        Object current = TheTarget;
        for(int i = 0; i < items.length - 1; i++) {
            current = getSingleProperty(current,items[i]);
            if(current == null) 
                throw new IllegalArgumentException("Property " + name + " cannot be set from Object " +
                    TheTarget);
        }
        setSingleProperty(current,name,value);
    }
    
    /**
        set the value for the property of a given property name
        @param Target non-null target object
        @param name non-null propertyName - must hand methods set<name>
        @param value possibly null new propetyValue - note if null and there are several set methods 
          any one might be called
        @throws IllegalArgumentException if elements of the chain are null
    */
    public static void setProperties(Object TheTarget,Map Values)
    {
        Collection keys = Values.keySet();
        String[] items = new String[keys.size()];
        keys.toArray(items);
        for(int i = 0; i < items.length; i++) {
            Object value = Values.get(items[i]);
            setSingleProperty(TheTarget,items[i],value);
        }
    }
    
    
    /**
        set the value for the property of a given property name
        @param Target non-null target object
        @param name non-null propertyName - must hand methods set<name>
        @param value possibly null new propetyValue - note if null and there are several set methods 
          any one might be called
    */
    public static void setSingleProperty(Object TheTarget,String name,Object value)
    {
        Class TestArgs = null;
        if(value != null) 
            TestArgs = value.getClass();
        Method TheMethod = findSetMethod(TheTarget.getClass(),name,false);
        if(TheMethod == null) {
            // repeat to debug
	    StringBuffer error = new StringBuffer(256);
	    String sep = System.getProperty("line.separator");
            error.append("Property setter for ").append(name).append(" in class " );
	    error.append(TheTarget.getClass().getName()).append(" not found");
            String[] Properties = getAllProperties(TheTarget.getClass());
            for(int i = 0; i < Properties.length; i++) 
                error.append(sep).append("  ").append(Properties[i]);
	    Assertion.logError(error.toString());
                
            TheMethod = findSetMethod(TheTarget.getClass(),name,false);
            throw new IllegalArgumentException("Property setter for " + name + " not found in class '" + 
                TheTarget.getClass().getName() + "\n" +
                "Available properties are: \n" + 
                Util.buildListString(Properties,1,8)
            );
        }
        Object ret = null;
            
        value = convertValueForMethod(TheMethod,value);
        Object[] args = { value };
            
        try {
            ret = TheMethod.invoke(TheTarget,args);         
        }
        catch(IllegalAccessException ex) {
            throw new IllegalArgumentException("IllegalAccessException for " + name);
        }
        catch(InvocationTargetException ex) {
	    // try {
            //    TheMethod.invoke(TheTarget,args);  // retry for trace
            //}
            //catch(Exception ex1) {
            //    ex1.printStackTrace();
	    // }
            throw new WrapperException(ex.getTargetException());
        }
    }
    /** 
    * find a method which matches the name name in class TheClass where data is the input
    * @param TheClass non-null Class to search for the method
    * @return if true return null of the method cannot be found
    */
    public static String[] getAllProperties(Class TheClass)
    {
        ClassAnalysis anal = getAnalysis(TheClass);
        Hashtable tb = anal.m_Properties;
        TreeSet holder = new TreeSet();
        Enumeration keys =tb.keys();
        while(keys.hasMoreElements())
            holder.add(keys.nextElement());
        
        String[] ret = new String[holder.size()];
        holder.toArray(ret);
        return(ret);
    }
    
    /** 
    * find a method which matches the name name in class TheClass where data is the input
    * @param TheClass non-null Class to search for the method
    * @param name non-nullmethod name search caseful and then caseless
    * @param data class of data to set
    * @param if true return null of the method cannot be found
    */
    public static Method findSetMethod(Class TheClass,String name,boolean nullOK)
    {
        ClassAnalysis anal = getAnalysis(TheClass);
        Hashtable tb = anal.m_Properties;
        ClassProperty prop = (ClassProperty)tb.get(name);
        if(prop == null) { // try caseless search
            prop = (ClassProperty)tb.get(name.toUpperCase());
        }
        if(prop != null) 
            return(prop.m_SetMethod);
        if(nullOK)
            return(null);
	StringBuffer error = new StringBuffer(256);
	String sep = System.getProperty("line.separator");
        error.append("Cannot find method to set '").append(name).append("' property in ").append(TheClass.getName());    
        error.append(sep).append("Methods are: ");
        Enumeration e = tb.keys();
        while(e.hasMoreElements()) {
	    error.append(sep).append("   ");
            error.append(e.nextElement().toString());
        }
        Assertion.logError(error.toString());  
        throw new IllegalArgumentException("Cannot find method to set '" + name + "' property in " + TheClass.getName());  
    }

    /**
        return a get method for a given property name
        @param TheClass non-null target class
        @param name non-null propertyName - must hand methods get<name>
        @return - possibly null Method
    */
    public static Method findGetMethod(Class TheClass,String name)
    {
        ClassAnalysis anal = getAnalysis(TheClass);
        Hashtable tb = anal.m_Properties;
        ClassProperty prop = (ClassProperty)tb.get(name);
        if(prop == null) { // try caseless search
            prop = (ClassProperty)tb.get(name.toUpperCase());
        }
        if(prop != null) 
            return(prop.m_GetMethod);
        return(null);
    }

 /* =========================================================
    protected code
    ========================================================= */
    
    protected static ClassAnalysis getAnalysis(Class in) 
    {
        // make sure basic tables built
        if(gClassToAnalysisTable == null) {
            buildSourceTable();
        }
        ClassAnalysis ret = (ClassAnalysis)gClassToAnalysisTable.get(in);
        if(ret != null)
            return(ret);
        return(analyzeClass(in));
    }
    
    protected static synchronized void buildSourceTable() 
    {
        if(gClassToAnalysisTable == null) {
            gClassToAnalysisTable = new Hashtable();
        }
    }
    
    protected static synchronized ClassAnalysis analyzeClass(Class in) 
    {
        ClassAnalysis ret = (ClassAnalysis)gClassToAnalysisTable.get(in);
        if(ret != null)
            return(ret);
        ret = new ClassAnalysis();  
        
        Method[] methods = in.getMethods();
        accumulateProperties(in,ret.m_Properties,methods);
        accumulateCollections(in,ret.m_Collections,methods);
        
        gClassToAnalysisTable.put(in,ret);
        return(ret);            
    }
    
    
    protected static synchronized void accumulateProperties(Class TheClass,Hashtable items,Method[] methods)
    {
         for(int i = 0; i < methods.length; i++) {
            // HACK - ignore methods which are not get and set 
            // possibilities i.e. CDGJColllection.add(Object,String) S. Lewis
            if(methods[i].getParameterTypes().length <= 1) {
                String testName = methods[i].getName();
                testName = null;
                if(ClassUtilities.isGetMethod(methods[i],null)) {
                    handlePossibleProperty(TheClass,methods[i],methods,items);
                }
            }
        }
    }
    
    
    protected static synchronized void accumulateCollections(Class TheClass,Hashtable items,Method[] methods)
    {
        Vector possibleCollections = new Vector();
        
        for(int i = 0; i < methods.length; i++) {
            // HACK - ignore methods which are not get and set 
            // possibilities i.e. CDGJColllection.add(Object,String) S. Lewis
            if(methods[i].getParameterTypes().length == 1) {
                String testName = methods[i].getName();
                if(testName.startsWith("add") && !testName.endsWith("Listener") ) 
                    possibleCollections.addElement(testName.substring(3));
            }
        }
       for(int i = 0; i < possibleCollections.size(); i++) {
            String ColectionName = (String)possibleCollections.elementAt(i);
            ClassCollection TheCollection = buildCollection(TheClass,ColectionName,methods);
            if(TheCollection != null && validateClassCollection(TheCollection)) {
                items.put(ColectionName,TheCollection);
                items.put(ColectionName.toUpperCase(),TheCollection);
            }
       }
        
    }

    protected static ClassCollection buildCollection(Class TheClass,String Name,Method[] methods)
    {
        String AddName = "add" + Name;
        String CountName = "get" + Name + "Count";
        String RemoveName = "remove" + Name;
        String EnumerateName = "enumerate" + Name;
        String IterateName = "iterate" + Name;
        
        ClassCollection ret = new ClassCollection();
        ret.m_Class = TheClass;
        ret.m_Name = Name;
        
        for(int i = 0; i < methods.length; i++) {
            if(methods[i].getParameterTypes().length == 1) {
                String testName = methods[i].getName();
                if(testName.equals(AddName)) {
                    ret.m_AddMethod = methods[i];
                    ret.m_Type = methods[i].getParameterTypes()[0];
                }
                if(testName.equals(RemoveName)) {
                    ret.m_RemoveMethod = methods[i];
                }
            }
            if(methods[i].getParameterTypes().length == 0) {
                String testName = methods[i].getName();
                if(testName.equals(CountName)) {
                    if(methods[i].getReturnType() == Integer.TYPE)
                       ret.m_CountMethod = methods[i];
                }
                if(testName.equals(EnumerateName)) {
                    if(methods[i].getReturnType() == Enumeration.class)
                        ret.m_EnumerateMethod = methods[i];
                }
                if(testName.equals(IterateName)) {
                    if(methods[i].getReturnType() == Iterator.class)
                        ret.m_EnumerateMethod = methods[i];
                }
            }
        }
        return(ret);
        
    }
    
    
    protected static void handlePossibleProperty(Class TheClass,Method test,Method[] methods,Hashtable tb)
    {
        Method SetMethod = null;
        String testName = test.getName();
        String propName;
        if(testName.equals("getClass"))
            return;
        String TestSet;
        if(testName.startsWith("get")) {
            propName = testName.substring(3);
        }
        else {
             propName =  testName.substring(2);
       }
       TestSet = "set" + propName;
       for(int i = 0; i < methods.length; i++) {
         if(methods[i].getName().equals(TestSet)) {
            SetMethod = methods[i];
            break;
         }
       }
    //   if(SetMethod == null)
    //        return;
       ClassProperty prop = new  ClassProperty();
       prop.m_Name = propName;
       prop.m_Class = TheClass;
       prop.m_GetMethod = test;
       prop.m_SetMethod = SetMethod;
       
       tb.put(propName,prop);
       tb.put(propName.toUpperCase(),prop);
       
    }
    
    public static Object convertValueForMethod(Method TheMethod,Object value)
    {
        Class RequiredClass = TheMethod.getParameterTypes()[0];
        return(convertValueToClass(RequiredClass,value));
    }
    
    
    protected static Object convertStringToDesiredType(Class dataType,String sValue)
    {
		Object value = null;
	    dataType = ClassUtilities.primitiveToWrapper(dataType);
		if(dataType == String.class) {
			value = sValue;
		} else if(dataType == Integer.class) {
			value = new Integer(sValue);
		} else if(dataType == Boolean.class) {
		    // Accept "t" as true
		    if(sValue.equals("t"))
		        value = new Boolean(true);
		    else
			    value = new Boolean(sValue);
		} else if(dataType == Short.class) {
			value = new Short(sValue);
		} else if(dataType == Character.class) {
		    if(sValue.length() > 0)
			    value = new Character(sValue.charAt(0));
			else
			    value = new Character(' ');
		} else if(dataType == Byte.class) {
			value = new Byte(sValue);
		} else if(dataType == Double.class) {
			value = new Double(sValue);
		} else if(dataType == Float.class) {
			value = new Float(sValue);
		} else if(dataType == Long.class) {
			value = new Long(sValue);
		} else if(dataType == Class.class) {
		    try {
			    value = Class.forName(sValue);
			}
			catch(Exception ex) {
			    throw new IllegalArgumentException("Bad class name " + sValue);
			}
			
		} else if(dataType == java.util.Date.class) {
			value = new java.util.Date(Long.parseLong(sValue));
		} else if(dataType == java.lang.Object.class) {
			value = sValue;
		} else {
		    // find a constructor taking a string and call it
		    Class[] args = { String.class };
		    try {
		        Constructor Const = dataType.getConstructor(args);
		        Object[] constArgs = { sValue };
		        value = Const.newInstance(constArgs);
		    }
		    catch(Exception ex) {
			    throw new IllegalArgumentException("Data cannot be constructed from String: " + 
			        dataType.getName() + " String = " + sValue);
			}
		}
		return(value);
    }
            
		
    
    public static Object convertValueToClass(Class RequiredClass,Object value)
    {
        if(value == null)
            return(value); // no conversion needed;
        if(RequiredClass.isInstance(value))
            return(value); // no conversion needed;
        Class ValueClass = value.getClass();
        ValueClass = null;
        if(value instanceof String) {
            return(convertStringToDesiredType(RequiredClass,(String)value));
        }
        if(RequiredClass.isPrimitive()) {
            return(convertPrimitive(RequiredClass, value));
        }
        if(Number.class.isAssignableFrom(RequiredClass)) {
            return(convertNumber(RequiredClass, value));
        }
        if(RequiredClass == String.class)
            return(value.toString());
        
        return(handleBadConversion(RequiredClass, value)); // cannot handle this case
    }
    
    protected static Object handleBadConversion(Class RequiredClass,Object value)
    {
        if(gConverter != null) {
            return(gConverter.convertToDesiredClass(value,RequiredClass));
        }
        
        throw new IllegalArgumentException("Cannot convert " + value + 
            " of class " + value.getClass().getName() +
            " to " + RequiredClass.getName());
    }

    protected static Object convertNumber(Class RequiredClass,Object avalue)
    {
        if(avalue instanceof Number) {
            Number RealValue = (Number)avalue;
            if(RequiredClass == Integer.class) {
                return(new Integer(RealValue.intValue()));
            }
            if(RequiredClass == Double.class) {
                return(new Double(RealValue.doubleValue()));
            }
            if(RequiredClass == Short.class) {
                return(new Short((short)RealValue.intValue()));
            }
            if(RequiredClass == Long.class) {
                return(new Long(RealValue.longValue()));
            }
        }
        return(handleBadConversion(RequiredClass, avalue)); // cannot handle this case
     }
    
    protected static Object convertPrimitive(Class RequiredClass,Object value)
    {
        Class ValueClass = value.getClass();
    
        // Check Obviove conversion
        if(RequiredClass == Boolean.TYPE) {
            if(ValueClass == Boolean.class) return(value);
            return(handleBadConversion(RequiredClass, value)); // cannot handle this case
        }
        if(RequiredClass == Integer.TYPE) {
            if(ValueClass == Integer.class) return(value);
            if(ValueClass == Double.class)
                return(new Integer((int)((Double)value).intValue()));
            return(handleBadConversion(RequiredClass, value)); // cannot handle this case
        }
        if(RequiredClass == Double.TYPE) {
            if(ValueClass == Double.class) return(value);
            if(ValueClass == Integer.class)
                return(new Double((double)((Integer)value).intValue()));
        }
        if(RequiredClass == Short.TYPE) {
            if(ValueClass == Short.class) return(value);
            return(handleBadConversion(RequiredClass, value)); // cannot handle this case
        }
        if(RequiredClass == Byte.TYPE) {
            if(ValueClass == Byte.class) return(value);
            return(handleBadConversion(RequiredClass, value)); // cannot handle this case
        }
        if(RequiredClass == Character.TYPE) {
            if(ValueClass == Character.class) return(value);
            return(handleBadConversion(RequiredClass, value)); // cannot handle this case
        }
        if(RequiredClass == Long.TYPE) {
            if(ValueClass == Long.class) return(value);
            return(handleBadConversion(RequiredClass, value)); // cannot handle this case
        }
        if(RequiredClass == Float.TYPE) {
            if(ValueClass == Float.class) return(value);
            return(handleBadConversion(RequiredClass, value)); // cannot handle this case
        }
            
        return(handleBadConversion(RequiredClass, value)); // cannot handle this case
    }
    
    
    protected static class ClassAnalysis
    {
        public Hashtable m_Properties;
        public Hashtable m_Collections;
        public ClassProperty[] m_PropertiesList;
        public ClassCollection[] m_CollectionsList;
        
        protected ClassAnalysis() 
        {
            m_Properties = new Hashtable();
            m_Collections = new Hashtable();
        }
        
        protected synchronized void buildCollectionsList()
        {
            if(m_CollectionsList != null)
                return;
            Set items = new HashSet();
            Enumeration e = m_Collections.keys();
            while(e.hasMoreElements()) {
                items.add(e.nextElement());
            }
            ClassCollection[] data = new ClassCollection[items.size()];
            items.toArray(data);
            m_CollectionsList = data;
        }
        
        protected synchronized void buildPropertiesList()
        {
            if(m_PropertiesList != null)
                return;
            Set items = new HashSet();
            Iterator e = m_Properties.values().iterator();
            while(e.hasNext()) {
                items.add(e.next());
            }
            ClassProperty[] data = new ClassProperty[items.size()];
            items.toArray(data);
            m_PropertiesList = data;
        }
    }
    
    public static void setAttributes(Object item,NameValue[] attributes)
    {
         for(int i = 0; i < attributes.length; i++) {
            String TestName = attributes[i].m_Name;
            // class is a special case - needed by xml
            if(TestName.equals("class")) {
                continue;
            }
            if(TestName.equals(XMLUtil.TAG_HANDLED)) {
                continue;
            }
            if(TestName != null && TestName.length() > 0)
                setProperty(item,attributes[i].m_Name,attributes[i].m_Value);
         }
    }
    
    
    
    protected static class IteratorToEnumeration implements Enumeration
    {
        private Iterator m_It;
        public boolean hasMoreElements() {
            return(m_It.hasNext());
        }
        public Object nextElement() {
            return(m_It.next());
        }
        protected IteratorToEnumeration(Iterator it)
        {
            m_It = it;
        }
    }
        
    public static boolean validateClassCollection(ClassCollection c)
    {
        if(c.m_AddMethod == null) {
            Assertion.logError("No add method for " + c.m_Name +
               " collection in class " + c.m_Class.getName());
               return(false);
        }
        if(c.m_RemoveMethod == null) {
        //    Assertion.logError("No remove method for " + c.m_Name +
         //      " collection, in class " + c.m_Class.getName());
               return(false);
        }
     /*   if(c.m_EnumerateMethod == null) {
            Assertion.logError("No enumerate method for " + c.m_Name +
               " collection in, class " + c.m_Class.getName());
               return(false);
        }
        */
        if(c.m_CountMethod == null) {
            Assertion.logError("No count method for " + c.m_Name +
               " collection, in class " + c.m_Class.getName());
               return(false);
        }
        return(true);
        
    }
    
}
